@@ -7,12 +7,12 @@ import (
 	"time"
 
 	v1 "k8s.io/api/core/v1"
-	kapierrors "k8s.io/apimachinery/pkg/api/errors"
-	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/types"
 	utilnet "k8s.io/apimachinery/pkg/util/net"
 	utilruntime "k8s.io/apimachinery/pkg/util/runtime"
 	utilwait "k8s.io/apimachinery/pkg/util/wait"
+	"k8s.io/apiserver/pkg/server/mux"
+	"k8s.io/apiserver/pkg/server/routes"
 	"k8s.io/client-go/kubernetes/scheme"
 	kv1core "k8s.io/client-go/kubernetes/typed/core/v1"
 	"k8s.io/client-go/tools/record"
@@ -24,15 +24,15 @@ import (
 	"k8s.io/kubernetes/pkg/proxy/iptables"
 	"k8s.io/kubernetes/pkg/proxy/metrics"
 	"k8s.io/kubernetes/pkg/proxy/userspace"
+	proxyutiliptables "k8s.io/kubernetes/pkg/proxy/util/iptables"
 	utiliptables "k8s.io/kubernetes/pkg/util/iptables"
-	utilnode "k8s.io/kubernetes/pkg/util/node"
 	utilsysctl "k8s.io/kubernetes/pkg/util/sysctl"
 	utilexec "k8s.io/utils/exec"
 
 	sdnproxy "github.com/openshift/sdn/pkg/network/proxy"
 	"github.com/openshift/sdn/pkg/network/proxyimpl/hybrid"
 	"github.com/openshift/sdn/pkg/network/proxyimpl/unidler"
-	"github.com/prometheus/client_golang/prometheus"
+	"k8s.io/component-base/metrics/legacyregistry"
 	"k8s.io/klog"
 )
 
@@ -64,34 +64,37 @@ func (sdn *OpenShiftSDN) runProxy(waitChan chan<- bool) {
 	if bindAddr.To4() == nil {
 		protocol = utiliptables.ProtocolIpv6
 	}
+	nodeAddr := bindAddr
+	if nodeAddr.Equal(net.IPv4zero) {
+		nodeAddr = net.ParseIP(sdn.nodeIP)
+		if nodeAddr == nil {
+			klog.Fatalf("Unable to parse node IP %q", sdn.nodeIP)
+		}
+	}
 
 	portRange := utilnet.ParsePortRangeOrDie(sdn.ProxyConfig.PortRange)
 
-	hostname, err := utilnode.GetHostname(sdn.ProxyConfig.HostnameOverride)
-	if err != nil {
-		klog.Fatalf("Unable to get hostname: %v", err)
-	}
-
 	eventBroadcaster := record.NewBroadcaster()
 	eventBroadcaster.StartRecordingToSink(&kv1core.EventSinkImpl{Interface: sdn.informers.KubeClient.CoreV1().Events("")})
-	recorder := eventBroadcaster.NewRecorder(scheme.Scheme, v1.EventSource{Component: "kube-proxy", Host: hostname})
+	recorder := eventBroadcaster.NewRecorder(scheme.Scheme, v1.EventSource{Component: "kube-proxy", Host: sdn.nodeName})
 
 	execer := utilexec.New()
 	iptInterface := utiliptables.New(execer, protocol)
 
 	var proxier proxy.Provider
-	var healthzServer *healthcheck.HealthzServer
+	var healthzServer healthcheck.ProxierHealthUpdater
 	if len(sdn.ProxyConfig.HealthzBindAddress) > 0 {
 		nodeRef := &v1.ObjectReference{
 			Kind:      "Node",
-			Name:      hostname,
-			UID:       types.UID(hostname),
+			Name:      sdn.nodeName,
+			UID:       types.UID(sdn.nodeName),
 			Namespace: "",
 		}
-		healthzServer = healthcheck.NewDefaultHealthzServer(sdn.ProxyConfig.HealthzBindAddress, 2*sdn.ProxyConfig.IPTables.SyncPeriod.Duration, recorder, nodeRef)
+		healthzServer = healthcheck.NewProxierHealthServer(sdn.ProxyConfig.HealthzBindAddress, 2*sdn.ProxyConfig.IPTables.SyncPeriod.Duration, recorder, nodeRef)
 	}
 
 	enableUnidling := false
+	var err error
 
 	switch string(sdn.ProxyConfig.Mode) {
 	case "unidling+iptables":
@@ -99,17 +102,22 @@ func (sdn *OpenShiftSDN) runProxy(waitChan chan<- bool) {
 		fallthrough
 	case "iptables":
 		klog.V(0).Infof("Using %s Proxier.", sdn.ProxyConfig.Mode)
-		if bindAddr.Equal(net.IPv4zero) {
-			var err error
-			bindAddr, err = getNodeIP(sdn.informers.KubeClient.CoreV1(), hostname)
-			if err != nil {
-				klog.Fatalf("Unable to get a bind address: %v", err)
-			}
-		}
 		if sdn.ProxyConfig.IPTables.MasqueradeBit == nil {
 			// IPTablesMasqueradeBit must be specified or defaulted.
 			klog.Fatalf("Unable to read IPTablesMasqueradeBit from config")
 		}
+
+		var localDetector proxyutiliptables.LocalTrafficDetector
+		if sdn.ProxyConfig.ClusterCIDR == "" {
+			klog.Warningf("Kubeproxy does not support multiple cluster CIDRs, configuring no-op local traffic detector")
+			localDetector = proxyutiliptables.NewNoOpLocalDetector()
+		} else {
+			localDetector, err = proxyutiliptables.NewDetectLocalByCIDR(sdn.ProxyConfig.ClusterCIDR, iptInterface)
+			if err != nil {
+				klog.Fatalf("Unable to configure local traffic detector: %v", err)
+			}
+		}
+
 		proxier, err = iptables.NewProxier(
 			iptInterface,
 			utilsysctl.New(),
@@ -118,9 +126,9 @@ func (sdn *OpenShiftSDN) runProxy(waitChan chan<- bool) {
 			sdn.ProxyConfig.IPTables.MinSyncPeriod.Duration,
 			sdn.ProxyConfig.IPTables.MasqueradeAll,
 			int(*sdn.ProxyConfig.IPTables.MasqueradeBit),
-			sdn.ProxyConfig.ClusterCIDR,
-			hostname,
-			bindAddr,
+			localDetector,
+			sdn.nodeName,
+			nodeAddr,
 			recorder,
 			healthzServer,
 			sdn.ProxyConfig.NodePortAddresses,
@@ -181,7 +189,6 @@ func (sdn *OpenShiftSDN) runProxy(waitChan chan<- bool) {
 		proxier, err = hybrid.NewHybridProxier(
 			hp,
 			unidlingUserspaceProxy,
-			sdn.ProxyConfig.IPTables.SyncPeriod.Duration,
 			sdn.ProxyConfig.IPTables.MinSyncPeriod.Duration,
 			sdn.informers.KubeInformers.Core().V1().Services().Lister(),
 		)
@@ -213,11 +220,14 @@ func (sdn *OpenShiftSDN) runProxy(waitChan chan<- bool) {
 
 	// Start up a metrics server if requested
 	if len(sdn.ProxyConfig.MetricsBindAddress) > 0 {
-		mux := http.NewServeMux()
+		mux := mux.NewPathRecorderMux("kube-proxy")
 		mux.HandleFunc("/proxyMode", func(w http.ResponseWriter, r *http.Request) {
 			fmt.Fprintf(w, "%s", sdn.ProxyConfig.Mode)
 		})
-		mux.Handle("/metrics", prometheus.Handler())
+		mux.Handle("/metrics", legacyregistry.Handler())
+		if sdn.ProxyConfig.EnableProfiling {
+			routes.Profiling{}.Install(mux)
+		}
 		go utilwait.Until(func() {
 			err := http.ListenAndServe(sdn.ProxyConfig.MetricsBindAddress, mux)
 			if err != nil {
@@ -230,39 +240,3 @@ func (sdn *OpenShiftSDN) runProxy(waitChan chan<- bool) {
 	go utilwait.Forever(proxier.SyncLoop, 0)
 	klog.Infof("Started Kubernetes Proxy on %s", sdn.ProxyConfig.BindAddress)
 }
-
-// getNodeIP is copied from the upstream proxy config to retrieve the IP of a node.
-func getNodeIP(client kv1core.CoreV1Interface, hostname string) (net.IP, error) {
-	var node *v1.Node
-	var nodeErr error
-
-	// We may beat the thread that causes the node object to be created,
-	// so if we can't get it, then we need to wait.
-	// This will wait 0, 2, 4, 8, ... 64 seconds, for a total of ~2 mins
-	nodeWaitBackoff := utilwait.Backoff{
-		Duration: 2 * time.Second,
-		Factor:   2,
-		Steps:    7,
-	}
-	utilwait.ExponentialBackoff(nodeWaitBackoff, func() (bool, error) {
-		node, nodeErr = client.Nodes().Get(hostname, metav1.GetOptions{})
-		if nodeErr == nil {
-			return true, nil
-		} else if kapierrors.IsNotFound(nodeErr) {
-			klog.Warningf("waiting for node %q to be registered with master...", hostname)
-			return false, nil
-		} else {
-			return false, nodeErr
-		}
-	})
-	if nodeErr != nil {
-		return nil, fmt.Errorf("failed to retrieve node info (after waiting): %v", nodeErr)
-	}
-
-	nodeIP, err := utilnode.GetNodeHostIP(node)
-	if err != nil {
-		return nil, fmt.Errorf("failed to retrieve node IP: %v", err)
-	}
-
-	return nodeIP, nil
-}
